import subprocess
import os
import glob
import time

def test_modelchoice(path):
    """Run basic Model choice example
    """
    for filePath in glob.glob('modelchoice_out.*'):
        os.remove(filePath)
    subprocess.run(path)

def test_modelchoice_multi(path):
    """Run basic multi target Model choice example
    """
    for filePath in glob.glob('modelchoice_out.*'):
        os.remove(filePath)
    subprocess.call([path,"-b","statobsRF2.txt"])

def test_estimparam(path):
    """Run basic Parameter estimation example
    """
    for filePath in glob.glob('estimparam_out.*'):
        os.remove(filePath)
    subprocess.call([path,"--parameter","ra","--chosenscen","3","--noob","50"])

def test_estimparam_multi(path):
    """Run basic multi target Parameter estimation example
    """
    for filePath in glob.glob('estimparam_out.*'):
        os.remove(filePath)
    subprocess.call([path,"-b","statobsRF2.txt","--parameter","ra","--chosenscen","3","--noob","50"])

def test_parallel(path):
    """Check multithreaded performance
    """
    for filePath in glob.glob('estimparam_out.*'):
        os.remove(filePath)
    time1 = time.time()
    subprocess.call([path,"--parameter","ra","--chosenscen","3","--noob","50","-j","1"])
    time2 = time.time()
    subprocess.call([path,"--parameter","ra","--chosenscen","3","--noob","50","-j","8"])
    time3 = time.time()
    assert (time2-time1) > 1.5 * (time3 - time2)



import subprocess
import os
import glob
import time
import h5py
import pyabcranger

import numpy as np

f = h5py.File('reftable.h5','r')
statobs = np.loadtxt('statobsRF.txt',skiprows=2)

stats_mc = np.array(f['stats'])
params_mc = np.array(f['params'])

rf_mc = pyabcranger.reftable(
    f['nrec'][0],
    f['nrecscen'],
    f['nparam'],
    f['params'].attrs['params_names'],
    f['stats'].attrs['stats_names'],
    stats_mc,
    params_mc,
    f['scenarios']
    )

selected = np.array(f['scenarios']) == 3
stats_ep = np.array(stats_mc[selected,:])
params_ep = np.array(params_mc[selected,:])

rf_ep = pyabcranger.reftable(
    np.sum(selected),
    f['nrecscen'],
    f['nparam'],
    f['params'].attrs['params_names'],
    f['stats'].attrs['stats_names'],
    stats_ep,
    params_ep,
    f['scenarios']
    )

def test_modelchoice():
    """Run basic Model choice example
    """
    for filePath in glob.glob('modelchoice_out.*'):
        os.remove(filePath)
    postres = pyabcranger.modelchoice(rf_mc, statobs,"",False)
    assert len(postres.votes[0]) == 6

def test_estimparam(path):
    """Run basic Parameter estimation example
    """
    for filePath in glob.glob('estimparam_out.*'):
        os.remove(filePath)
    pyabcranger.estimparam(rf_ep,statobs,"--parameter ra --chosenscen 3 --noob 50",False,False)

def test_parallel(path):
    """Check multithreaded performance
    """
    for filePath in glob.glob('estimparam_out.*'):
        os.remove(filePath)
    time1 = time.time()
    pyabcranger.estimparam(rf_ep,statobs,"--parameter ra --chosenscen 3 --noob 50 --threads 1",False,False)
    time2 = time.time()
    pyabcranger.estimparam(rf_ep,statobs,"--parameter ra --chosenscen 3 --noob 50 --threads 8",False,False)
    time3 = time.time()
    assert (time2-time1) > 1.5 * (time3 - time2)



from cmaketools import setup
import sys
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

python_version = str(sys.version_info.major) + "." + str(sys.version_info.minor)
print("FUCKING PYTHON VERSION : " + python_version)

if((len(sys.argv) > 1) and (sys.argv[1] == "sdist")):
    configure_opts = []
elif sys.platform == "linux":
    configure_opts = ["-DPYTHON_EXECUTABLE="+sys.executable,"-DPYBIND11_PYTHON_VERSION="+python_version,"-DPYABCRANGER=TRUE","-DUSE_MKL:BOOL=TRUE","-DMAKE_STATIC_EXE:BOOL=TRUE","-DLAPACK_ROOT:STRING=/opt/intel/mkl/lib/intel64","-DLAPACK_LIBRARIES:STRING=-Wl,--start-group /opt/intel/mkl/lib/intel64/libmkl_intel_lp64.a /opt/intel/mkl/lib/intel64/libmkl_tbb_thread.a /opt/intel/mkl/lib/intel64/libmkl_core.a -Wl,--end-group;pthread;m;dl","-DBLAS_LIBRARIES:STRING=-Wl,--start-group /opt/intel/mkl/lib/intel64/libmkl_intel_lp64.a /opt/intel/mkl/lib/intel64/libmkl_tbb_thread.a /opt/intel/mkl/lib/intel64/libmkl_core.a -Wl,--end-group;pthread;m;dl"]
elif sys.platform == "darwin":
    configure_opts = ["-DPYTHON_EXECUTABLE="+sys.executable,"-DPYBIND11_PYTHON_VERSION="+python_version,"-DPYABCRANGER=TRUE","-DUSE_MKL:BOOL=FALSE","-DCMAKE_BUILD_TYPE:STRING=Release"]
elif sys.platform == "win32":
    configure_opts = ["-DPYTHON_EXECUTABLE="+sys.executable,"-DPYBIND11_PYTHON_VERSION="+python_version,"-DPYABCRANGER=TRUE","-DUSE_MKL:BOOL=FALSE","-DMAKE_STATIC_EXE:BOOL=TRUE","-DVCPKG_TARGET_TRIPLET:STRING=x64-windows-static"]
else:
    exit(1)

setup(
    name="pyabcranger",
    version="0.0.49",
    author="François-David Collin",
    author_email="fradav@gmail.com",
    description="ABC random forests for model choice and parameter estimation, python wrapper",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/diyabc/abcranger",
    license="MIT License",
    src_dir="src",
    ext_module_hint=r"pybind11_add_module",
    generator="Ninja",
    config="Release",
    has_package_data=False,
    packages=["pyabcranger"],
    configure_opts=configure_opts,
    build_opts=["--target","pyabcranger"]
)